-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, npcg = ...

local functua 	= npcg.functua
local util 			= npcg.functua.util

local generator = { }
local data      = { }
local tests     = { }

-- generators and data do not exist yet
npcg.generators.age	= generator
npcg.data.age				= data
npcg.tests.age			= tests

local inc = util.getIncremental(1)

local ages = { }	-- Redo this to depending on race.
ages[       "child"] = { tags = {     "minor" }, sortOrder = inc(), description = "A young person without any real responsibilities"                               }
ages[        "teen"] = { tags = { "workforce" }, sortOrder = inc(), description = "No longer a child, but not concidered adult by their race yet"                  }
ages[ "young adult"] = { tags = { "workforce" }, sortOrder = inc(), description = "Recently accepted as adult by their race"                                       }
ages[       "adult"] = { tags = { "workforce" }, sortOrder = inc(), description = "Fully grown and with responsibilities for themself or others"                   }
ages["mature adult"] = { tags = { "workforce" }, sortOrder = inc(), description = "In the prime of their life. Family normally established already"                }
ages[       "elder"] = { tags = { "workforce" }, sortOrder = inc(), description = "Strenght and stamina might have started to decline"                             }
ages[         "old"] = { tags = { "workforce" }, sortOrder = inc(), description = "Often retired. The time for possible grandchildren. Often affected by old age"  }
ages[   "venerable"] = { tags = { "workforce" }, sortOrder = inc(), description = "Life is slowly comming to an end. Heavily affected by old age"                  }

-- generateRandomAge :: Race a => a -> Age
function generateRandomAge(characterTemplate)
  return util.randomFrom(
    util.eitherOr(
      characterTemplate:getRace()   	== "forsaken" 		or -- Make this teen
				characterTemplate:getRace() 	== "death knight" or
				characterTemplate:getRace() 	== "illidari" 		or
				characterTemplate:getRace() 	== "void elf" 		or
				characterTemplate:getTribe()	== "lightforged", 
      
			data.olderThan("young adult"),
      functua.table.keys(ages)
    )
  ) or ""
end

-- What are these doing here?!
-- keysBy :: (a -> k) -> { b, a } -> { k, b }
local function keysBy(sortF, xs)
  local output = { }
  
  for k,v in pairs(xs) do
    output[sortF(v)] = k
  end
  return output
end

-- olderThan :: Age -> { Age }
-- Returns all ages older than a given age (including the given age).
function data.olderThan(age)
  return functua.list.skipWhile(function(x) return x == age end, keysBy(function(v) return v.sortOrder end, ages))
end

-- youngerThan :: Age -> { Age }
function data.youngerThan(age)
  return functua.list.takeWhile(function(x) return x == age end, keysBy(function(v) return v.sortOrder end, ages))
end

-- agesBetween :: Age -> { Age }
function data.agesBetween(min, max)
  local output = functua.list.skipWhile(function(x) return x == min end, data.youngerThan(max))
  table.insert(output, max)
  return output
end

function data.getAgeDescription(age)
  return ages[age].description
end

-- End of File