-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, npcg = ...

local functua 	= npcg.functua
local util 			= npcg.functua.util

local generator = { }
local data      = { }
local tests     = { }

-- generators and data do not exist yet
npcg.generators.alignment	= generator
npcg.data.alignment				= data
npcg.tests.alignment			= tests

-- Adding "any" and another will always return true.

local alignments = { }
alignments["lawful good"    ] = { moral = {    "good", "any" }, ethic = {  "lawful", "any" }, description = "Honor and compassion. Tells the truth and helps others. The Paladin"                          }
alignments["lawful neutral" ] = { moral = { "neutral", "any" }, ethic = {  "lawful", "any" }, description = "Doing good without bias for or against order. The Benefactor"                                 }
alignments["lawful evil"    ] = { moral = {    "evil", "any" }, ethic = {  "lawful", "any" }, description = "A free spirit. Belives in good, but disregards the law. The Rebel"                            }
alignments["neutral good"   ] = { moral = {    "good", "any" }, ethic = { "neutral", "any" }, description = "Order and Organisation. Reliable and honorable. The Judge"                                    }
alignments["true neutral"   ] = { moral = { "neutral", "any" }, ethic = { "neutral", "any" }, description = "Acts without prejudice. Lack of conviction. The Undecided"                                    }
alignments["neutral evil"   ] = { moral = {    "evil", "any" }, ethic = { "neutral", "any" }, description = "True freedom. Avoids authorities. The Free spirit"                                            }
alignments["chaotic good"   ] = { moral = {    "good", "any" }, ethic = { "chaotic", "any" }, description = "Plays by the rules. Lacks mercy and compassion. Intentionally evil. The Dominator"            }
alignments["chaotic neutral"] = { moral = { "neutral", "any" }, ethic = { "chaotic", "any" }, description = "No regards for others. Doesnt follow laws or traditions. Evil without honor. The Malefactor"  }
alignments["chaotic evil"   ] = { moral = {    "evil", "any" }, ethic = { "chaotic", "any" }, description = "Hot tempered, vicious and unpredictable. Demonic. Driven by destruction. The Destroyer"       }

local alignmentsByTag = { 
  ["vengeful"] = {
    "chaotic neutral",
    "lawful neutral",
    "lawful evil",
    "neutral evil",
    "chaotic evil" },
}

-- generateRandomAlignment :: CharacterTemplate -> Moral -> Alignment
function generateRandomAlignment(characterTemplate, empty, empty2, moral)
	return util.randomFrom(
		functua.table.filterKeysByValue(
			function(val) return
				functua.table.contains(moral or "any",	val.moral) --[[or contains(params.moral, val.moral)]]
			end,
			alignments
		)
	) or "Error: No options found!"
end

-- alignmentsWithMoral :: Moral -> { Alignment }
function data.alignmentsWithMoral(moral)
  return functua.table.filterKeysByValue(
    function(val) return
      functua.table.contains(moral, val.moral)
    end,
    alignments
  )
end

function data.alignmentsByEthics(ethic)
  return functua.table.filterKeysByValue(
    function(val) return
      functua.table.contains(ethic, val.ethic)
    end,
    alignments
  )
end

-- allAlignmentsExcept :: Alignment -> ... -> { Alignment }
function data.allAlignmentsExcept(...)
  local args = {...}
  return functua.table.without(functua.table.keys(alignments), args)
end

function data.getAlignmentsByTag(tag)
  return alignmentsByTag[tag] or functua.keys(alignments)
end

-- getAlignmentDescription :: string -> string
-- Returns a description belonging to a given alignment.
function data.getAlignmentDescription(alignment)
  return alignments[alignment].description
end

-- End of File