-------------------------------------------------------------------------------------
-- | Namespace Definitions |---------------------------------------------------------
-------------------------------------------------------------------------------------

-- Retrieve the addon name and the namespace table.
local addon, npcg = ...

-- Import Modules.
local data      = npcg.data.tribe
local trait     = npcg.generators.trait

-- Set up Modules.
local generator = { }

-- Make Modules public in npcg.
npcg.generators.tribe2 = generator

-------------------------------------------------------------------------------------
-- | Generator | --------------------------------------------------------------------
-------------------------------------------------------------------------------------

-- Default generator.
generator.generateRandomTrait = function() return "Error: generateRandomTrait is not set" end

-------------------------------------------------------------------------------------
-- | Generator Functions | ----------------------------------------------------------
-------------------------------------------------------------------------------------

-- Tribe is a strict subtrait, meaning it only has value in relation to it's dependency.
-- As such, the data tables are set up differently.

-- Type GenerationFunction :: DataTable -> ZoneDataTable -> SubZoneDataTable -> CharacterTemplate -> Zone -> SubZone -> DataTable
-- A generation function provides a DataTable based on the Zone, SubZone and CharacterTemplate, selected from the inputs.

-- checkAllLocations :: GenerationFunction
-- Returns a DataTable based on location settings.
local function checkAllLocations(defaultDataTable, zoneDataTable, subZoneDataTable, characterTemplate, zone, subZone)
  local race = characterTemplate.race
  return 
  -- Test whether the SubZone has values.
    ((subZoneDataTable[zone] or { })[subZone])[race]
    -- Test whether the Zone has values.
    or (zoneDataTable[zone] or { })[race]
    -- Fallback to default.
    or defaultDataTable[race]
    -- Fallback in case some extreme breaking happened.
    or { [""] = { weight = 1, traitTypes = { "retail", "classic" } } }
end

-- checkZonesOnly :: GenerationFunction
-- Returns a DataTable based on zone, but not subzone settings.
local function checkZonesOnly(defaultDataTable, zoneDataTable, subZoneDataTable, characterTemplate, zone)
  local race = characterTemplate.race
  return 
    -- Test whether the Zone has values.
    (zoneDataTable[zone] or { })[race]
    -- Fallback to default.
    or defaultDataTable[race]
    -- Fallback in case some extreme breaking happened.
    or { [""] = { weight = 1, traitTypes = { "retail", "classic" } } }
end

-- dontCheckLocations :: GenerationFunction
-- Returns a DataTable without checking locations.
local function dontCheckLocations(defaultDataTable, zoneDataTable, subZoneDataTable, characterTemplate)
  local race = characterTemplate.race
  return 
    -- Fallback to default.
    defaultDataTable[race]
    -- Fallback in case some extreme breaking happened.
    or { [""] = { weight = 1, traitTypes = { "retail", "classic" } } }
end

-- Returns the generatorF argument for createRaceGenerator based on the current options set in NPCG_Options.locationSettings
local function getTribeGeneratorFunction()
  if     not NPCG_Options.locationSettings.zones then
    return dontCheckLocations
  elseif not NPCG_Options.locationSettings.subZones then
    return checkZonesOnly
  else
    return checkAllLocations
  end
end

-------------------------------------------------------------------------------------
-- | Initialization | ---------------------------------------------------------------
-------------------------------------------------------------------------------------

-- initialize :: ()
-- Initializes the generator, supplying it with arguments such as options and data tables.
-- Function is impure w.r.t. generator.generateRandomTrait.
function generator.initialize()
  generator.generateRandomTrait = trait.createTraitGenerator(
    data.defaultTable,
    data.zoneDataTable,
    data.subZoneDataTable,
    getTribeGeneratorFunction(),
    trait.filterFunctions.getDefaultFilterFunction(),
    trait.selectorFunctions.selectUsingWeightedRandom)
end

-- End of File.